/**
 * Redistribution and use of this software and associated
 * documentation ("Software"), with or without modification, are
 * permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright statements
 *    and notices.  Redistributions must also contain a copy of this
 *    document.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * 3. The names "TRANSORG" or "EUTIST-AMI" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of one of the parties involved in the TRANSORG project. For
 * 	  written permission, please contact diasz@ida.liu.se.
 *
 * 4. Products derived from this Software may not be called "TRANSORG"
 *    nor may "TRANSORG" appear in their names without prior written
 *    permission.
 *
 * 5. Due credit should be given to the TRANSORG Project
 *    (http://www.ida.liu.se/~rtslab/TRANSORG/).
 *
 * THIS SOFTWARE IS PROVIDED BY THE TRANSORG PROJECT CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE TRANSORG PROJECT CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Copyright 2003 (C) TRANSORG Project Contributors All Rights Reserved.
 *
 * $Id: ComputeStatistics.java,v 1.0   Exp $
 *
 */

/** Author		: Diana Sznetivanyi
* Contact		: diasz@ida.liu.se
*/

/** Class used to parse files with timing values corresponding to the different server method calls used in the experiments, and the
* different segments from the request's path from the client to the server. For example one such file could be called a_method_SCL containing
* times spent in the client interceptor (SCL means Stay at CLient interceptor) by requests corresponding to the method call a_method
* performed during the experiments (for example the file could contain 200 such values)
* @author	<a href="mailto:diasz@ida.liu.se"> Diana Szentivanyi </a>
* @version $Revision: 1.0 $ $Date:2003/06/23$
*/

class DataSegm{
  long[] data;
  String segm;

  public DataSegm(long[] data,String segm){
    this.data=new long[data.length];
    System.arraycopy(data,0,this.data,0,data.length);
    this.segm=segm;
  }
}

class StatRec{
    int nr_samples;
  double st_dev;
  float mean;
  String segm;

  public StatRec(double st_dev,float mean,String segm,int nr_samples){
    this.st_dev=st_dev;
    this.mean=mean;
    this.segm=segm;
    this.nr_samples=nr_samples;
  }
}
public class ComputeStatistics {
  public static void main(String args[]){
    java.io.RandomAccessFile to_read[],to_write;
    int i,j,k,l;
    String segm,one_oper=null;
    float mean,mean_w_w;
    double st_dev,st_dev_w_w;
    long data_rtt_w_w[],data_rtt[],data_tw[],data[],data_w_sl[];
    java.util.Hashtable data_stat=new java.util.Hashtable();
    String path=args[0];

    int nr_l_s=Integer.parseInt(args[3]);
    if(args[4].equals("-opname"))
	one_oper=args[5];

    try{
      java.io.File file=new java.io.File(path+args[1]);
      String names[]=file.list(),op_name;
      java.util.Vector crt;
      Object an_elem;
      for(j=0;j<names.length;j++){
	  System.out.println("Parsing "+names[j]);
	  if((one_oper!=null&&names[j].startsWith(one_oper))||one_oper==null){
	      k=names[j].lastIndexOf('_');
	      segm=names[j].substring(k+1);
	      op_name=names[j].substring(0,k);
	      crt=(java.util.Vector)data_stat.get(op_name);
	      if(crt==null){
		  crt=new java.util.Vector();
		  data_stat.put(op_name,crt);
	      }
	      data=Statistics.read_data_file(path+args[1]+names[j]);
	      if(args[2].equals("-largest"))
		  data_w_sl=Statistics.data_set_without_largest(data,nr_l_s);
	      else
		  data_w_sl=Statistics.data_set_without_smallest(data,nr_l_s);
	      mean=Statistics.mean(data_w_sl);
	      st_dev=Statistics.st_dev(data_w_sl,mean);
	      crt.addElement(new StatRec(st_dev,mean,segm,data_w_sl.length));
	      l=crt.size();
	      if(segm.equals("TW")){
		  data_tw=new long[data.length];
		  System.arraycopy(data,0,data_tw,0,data.length);
		  if(l>1){
		      if(crt.elementAt(0) instanceof long[])
			  crt.insertElementAt(data_tw,1);
		      else
			  crt.insertElementAt(data_tw,0);
		  }
		  else{
		      if((l>0 && crt.elementAt(0) instanceof long[])||l<=0)
			  crt.addElement(data_tw);
		      else
			  crt.insertElementAt(data_tw,0);
		  }
	      }

	      if(segm.equals("RTT")) {
		  data_rtt=new long[data.length];
		  System.arraycopy(data,0,data_rtt,0,data.length);
		  crt.insertElementAt(data_rtt,0);
		  if(crt.size()>2){
		      an_elem=crt.remove(crt.size()-1);
		      if(crt.elementAt(1) instanceof long[])
			  crt.insertElementAt(an_elem,2);
		      else
			  crt.insertElementAt(an_elem,1);
		  }
	      }
	  }
      }

      java.util.Set entries;
      java.util.Iterator e_iter;
      java.util.Map.Entry elem;
      entries=data_stat.entrySet();
      e_iter=entries.iterator();
      long[] dif_w_w;
      StatRec total,segm_stat;
      to_write=new java.io.RandomAccessFile(args[args.length-1],"rw");
      while(e_iter.hasNext()){
	  elem=(java.util.Map.Entry)e_iter.next();
	  op_name=(String)elem.getKey();
	  crt=(java.util.Vector)elem.getValue();
	  if(crt.size()<9) continue;
	  data_rtt=(long[])crt.elementAt(0);
	  data_tw=(long[])crt.elementAt(1);
	  data_rtt_w_w=Statistics.diff(data_rtt,data_tw);
	  if(args[2].equals("-largest"))
	      data_rtt_w_w=Statistics.data_set_without_largest(data_rtt_w_w,nr_l_s);
	  else
	      data_rtt_w_w=Statistics.data_set_without_smallest(data_rtt_w_w,nr_l_s);
	  mean_w_w=Statistics.mean(data_rtt_w_w);
	  st_dev_w_w=Statistics.st_dev(data_rtt_w_w,mean_w_w);
	  to_write.writeBytes(op_name+"   ");

	  total=(StatRec)crt.elementAt(2);
	  to_write.writeBytes(total.nr_samples+" "+(long)total.mean+" "+(long)total.st_dev+"\n");
      for(i=3;i<crt.size();i++){
	  segm_stat=(StatRec)crt.elementAt(i);
	  to_write.writeBytes(op_name+" "+segm_stat.nr_samples+" "+segm_stat.segm+" "+(long)segm_stat.mean+" "+(long)segm_stat.st_dev+" "+(long)Statistics.proc(segm_stat.mean,total.mean)+"%\n");
      }
      to_write.writeBytes("\n\n");
      }
    }catch(java.lang.Exception e){e.printStackTrace();}
  }
}
